@@ -27,6 +27,7 @@ import DataFrame.Internal.Expression (
 import DataFrame.Internal.Statistics
 
 import Control.Monad
+import Control.Monad.IO.Class
 import qualified Data.Char as Char
 import Data.Function
 import Data.Functor
@@ -37,6 +38,8 @@ import qualified Data.Text as T
 import Data.Time
 import qualified Data.Vector as V
 import qualified Data.Vector.Unboxed as VU
+import qualified DataFrame.IO.CSV as CSV
+import qualified DataFrame.IO.Parquet as Parquet
 import Debug.Trace (trace)
 import Language.Haskell.TH
 import qualified Language.Haskell.TH.Syntax as TH
@@ -44,7 +47,7 @@ import Text.Regex.TDFA
 import Prelude hiding (maximum, minimum)
 import Prelude as P
 
-infix 4 .==, .<, .<=, .>=, .>
+infix 4 .==, .<, .<=, .>=, .>, ./=
 infixr 3 .&&
 infixr 2 .||
 
@@ -90,6 +93,9 @@ mod = BinaryOp "mod" Prelude.mod
 (.==) :: (Columnable a, Eq a) => Expr a -> Expr a -> Expr Bool
 (.==) = BinaryOp "eq" (==)
 
+(./=) :: (Columnable a, Eq a) => Expr a -> Expr a -> Expr Bool
+(./=) = BinaryOp "eq" (/=)
+
 eq :: (Columnable a, Eq a) => Expr a -> Expr a -> Expr Bool
 eq = BinaryOp "eq" (==)
 
@@ -99,11 +105,15 @@ eq = BinaryOp "eq" (==)
 lt :: (Columnable a, Ord a) => Expr a -> Expr a -> Expr Bool
 lt = BinaryOp "lt" (<)
 
+-- TODO: Generalize this pattern for other equality functions.
 (.>) :: (Columnable a, Ord a) => Expr a -> Expr a -> Expr Bool
-(.>) = BinaryOp "gt" (>)
+(.>) (Lit l) (Lit r) = Lit (l > r)
+(.>) (Lit l) expr = UnaryOp ("gt " <> T.pack (show l)) (l >) expr
+(.>) expr (Lit r) = UnaryOp ("leq " <> T.pack (show r)) (r <=) expr
+(.>) l r = BinaryOp "gt" (>) l r
 
 gt :: (Columnable a, Ord a) => Expr a -> Expr a -> Expr Bool
-gt = BinaryOp "gt" (>)
+gt = (.>)
 
 (.<=) :: (Columnable a, Ord a, Eq a) => Expr a -> Expr a -> Expr Bool
 (.<=) = BinaryOp "leq" (<=)
@@ -156,7 +166,7 @@ maximum :: (Columnable a, Ord a) => Expr a -> Expr a
 maximum expr = AggReduce expr "maximum" Prelude.max
 
 sum :: forall a. (Columnable a, Num a) => Expr a -> Expr a
-sum expr = AggReduce expr "sum" (+)
+sum expr = AggFold expr "sum" 0 (+)
 
 sumMaybe :: forall a. (Columnable a, Num a) => Expr (Maybe a) -> Expr a
 sumMaybe expr = AggVector expr "sumMaybe" (P.sum . catMaybes . V.toList)
@@ -199,10 +209,11 @@ stddevMaybe expr = AggVector expr "stddevMaybe" (sqrt . variance' . optionalToDo
 zScore :: Expr Double -> Expr Double
 zScore c = (c - mean c) / stddev c
 
-pow :: (Columnable a, Num a) => Int -> Expr a -> Expr a
-pow 0 _ = Lit 1
-pow 1 expr = expr
-pow i expr = UnaryOp ("pow " <> T.pack (show i)) (^ i) expr
+pow :: (Columnable a, Num a) => Expr a -> Int -> Expr a
+pow _ 0 = Lit 1
+pow (Lit n) i = Lit (n ^ i)
+pow expr 1 = expr
+pow expr i = UnaryOp ("pow " <> T.pack (show i)) (^ i) expr
 
 relu :: (Columnable a, Num a) => Expr a -> Expr a
 relu = UnaryOp "relu" (Prelude.max 0)
@@ -358,6 +369,22 @@ typeFromString s = fail $ "Unsupported types: " ++ unwords s
 dropFirstAndLast :: [a] -> [a]
 dropFirstAndLast = reverse . drop 1 . reverse . drop 1
 
+declareColumnsFromCsvFile :: String -> DecsQ
+declareColumnsFromCsvFile path = do
+    df <- liftIO (CSV.readCsv path)
+    declareColumns df
+
+-- TODO: We don't have to read the whole file, we can just read the schema.
+declareColumnsFromParquetFile :: String -> DecsQ
+declareColumnsFromParquetFile path = do
+    df <- liftIO (Parquet.readParquet path)
+    declareColumns df
+
+declareColumnsFromCsvWithOpts :: CSV.ReadOptions -> String -> DecsQ
+declareColumnsFromCsvWithOpts opts path = do
+    df <- liftIO (CSV.readSeparated opts path)
+    declareColumns df
+
 declareColumns :: DataFrame -> DecsQ
 declareColumns df =
     let
